function KH = compute_permeability(lx,ly,z)
%% Initializations
[nely, nelx] = size(z);
dx = lx/nelx;
dy = ly/nely;
nel = nelx*nely;
%% Periodic boundary conditions (PBC)
nnPBC = nel; %Number of unique nodes
m_IdNodesPBC = reshape(1:nnPBC, nelx, nely); % Matrix of unique nodes
m_IdNodesPBC(end+1,:) = m_IdNodesPBC(1,:); % Numbering bottom nodes
m_IdNodesPBC(:,end+1) = m_IdNodesPBC(:,1); % Numbering right nodes
m_ConnectPBC = zeros(nel,4); % Matrix with element connectivity
m_ConnectPBC = [reshape(m_IdNodesPBC(2:end,1:end-1),nel,1)...
                   reshape(m_IdNodesPBC(2:end,2:end),nel,1)...
                   reshape(m_IdNodesPBC(1:end-1,2:end),nel,1)...
                   reshape(m_IdNodesPBC(1:end-1,1:end-1),nel,1)];
ndofPBC = 3*nnPBC;
%% Calculation of element matrices
[ke,ge,pe,fe,SN] = calculate_matrices(dx, dy);
%% Assembling the system of linear algebraic equations with triplets
disp('--- ASSEMBLY ---');
v_elemF = find(~z); % Vector containing only fluid elements 
nelemF = size(v_elemF,1);
m_dofF = [m_ConnectPBC(v_elemF,1)'*2-1; m_ConnectPBC(v_elemF,1)'*2; 
            m_ConnectPBC(v_elemF,2)'*2-1; m_ConnectPBC(v_elemF,2)'*2;
            m_ConnectPBC(v_elemF,3)'*2-1; m_ConnectPBC(v_elemF,3)'*2; 
            m_ConnectPBC(v_elemF,4)'*2-1; m_ConnectPBC(v_elemF,4)'*2;
            2*nel+m_ConnectPBC(v_elemF,1)'; %
            2*nel+m_ConnectPBC(v_elemF,2)';
            2*nel+m_ConnectPBC(v_elemF,3)';
            2*nel+m_ConnectPBC(v_elemF,4)'];
% dof 1 and 2 (velocities) of each node and dof 3 (pressure) of each node
iK = repelem(reshape(m_dofF(1:8,1:end),nelemF*8,1),8,1);
jK = reshape(repmat(m_dofF(1:8,1:end),8,1),nelemF*64,1);
iG = repelem(reshape(m_dofF(1:8,1:end),nelemF*8,1),4,1);
jG = reshape(repmat(m_dofF(9:12,1:end),8,1),nelemF*32,1);
iP = repelem(reshape(m_dofF(9:12,1:end),nelemF*4,1),4,1);
jP = reshape(repmat(m_dofF(9:12,1:end),4,1),nelemF*16,1);
iA = [iK;iG;jG;iP];
jA = [jK;jG;iG;jP];
clear iK iG iP jK jG jP 
ke = reshape(ke',64,1);
ge = reshape(ge',32,1);
pe = reshape(pe',16,1);
coeff = [repmat(ke,nelemF,1);repmat(ge,nelemF,1);repmat(ge,nelemF,1);-repmat(pe,nelemF,1)];
A = sparse(iA,jA,coeff);
clear iA jA coeff
iF = [reshape(m_dofF(1:2:8,1:end),nelemF*4,1); reshape(m_dofF(2:2:8,1:end),nelemF*4,1)];
jF = [ones(nelemF*4,1);2*ones(nelemF*4,1)];
sF = repmat(fe,nelemF*2,1);
F = sparse(iF,jF,sF,ndofPBC,2);
clear iF jF sF 
%% Solving system of equations
solveF_u = 1:nel*2;
nullnodes = [unique(m_ConnectPBC(find(z),1:4))];
nulldof = [nullnodes*2-1 nullnodes*2];
solveF_u(nulldof) = []; %Matrix reduction
solveF_p = (nel*2+unique(m_ConnectPBC(find(~z),1:4)))';
solveF = [solveF_u solveF_p];
clear solveF_u solveF_p nullnodes nulldof m_ConnectPBC
X = zeros(ndofPBC,2);
disp('--- SOLVE ---');
X(solveF,:) = A(solveF,solveF)\F(solveF,:);
%% Permeability Calculation
% sum of elements velocities in case 1
vel_c1 = SN*sum(reshape(X(m_dofF(1:8,:),1),8,nelemF),2); 
%sum of elements velocities in case 2
vel_c2 = SN*sum(reshape(X(m_dofF(1:8,:),2),8,nelemF),2); 
KH = [vel_c1'; vel_c2']/(nel*dx*dy);
disp(KH)
end

%% Element Matrices
function [k,g,pe,f,SN] = calculate_matrices(dx, dy)
  coordsElem = [0 0; dx 0; dx dy; 0 dy];
  rr = [-1/sqrt(3) 1/sqrt(3)];
  ww = [1 1];
  ss = rr;
  k = zeros(8);
  SN=zeros(2,8);
  C = [2 0 0; 0 2 0; 0 0 1];
  g = zeros(8,4);
  pe = zeros(4);
  h2 = ((dx)^2)+((dy)^2);
  stab = h2/12;
  f = zeros(4,1);
  for i = 1:2
    r = rr(1,i);
    for j = 1:2
      s = ss(1,j);
      N=(1/4)*[(1-s)*(1-r) 0 (1-s)*(1+r) 0 (1+s)*(1+r) 0 (1-r)*(1+s) 0;
                 0 (1-s)*(1-r) 0 (1-s)*(1+r) 0 (1+s)*(1+r) 0 (1-r)*(1+s)];  
      dN1dr = -1/4*(1-s);
      dN2dr = +1/4*(1-s);
      dN3dr = +1/4*(1+s);
      dN4dr = -1/4*(1+s);    
      dN1ds = -1/4*(1-r);
      dN2ds = -1/4*(1+r);
      dN3ds = +1/4*(1+r);
      dN4ds = +1/4*(1-r);
      DN = [dN1dr dN2dr dN3dr dN4dr;
             dN1ds dN2ds dN3ds dN4ds];
      J = DN*coordsElem;
      invJ = 1.0/det(J)*[J(2,2) -J(1,2); -J(2,1) J(1,1)];
      DNxy = invJ*DN;
      B = [DNxy(1,1) 0 DNxy(1,2) 0 DNxy(1,3) 0 DNxy(1,4) 0;
            0 DNxy(2,1) 0 DNxy(2,2) 0 DNxy(2,3) 0 DNxy(2,4);
            DNxy(2,1) DNxy(1,1) DNxy(2,2) DNxy(1,2) DNxy(2,3) DNxy(1,3) DNxy(2,4) DNxy(1,4)];
      k = k + (B')*C*B*det(J)*ww(1,i)*ww(1,j);
      g = g + (B')*[1;1;0]*N(1:4:end)*(det(J))*ww(1,i)*ww(1,j);
      Bp = inv(J)*[DN(1,:);DN(2,:)];
      pe = pe + stab*(Bp')*Bp*(det(J))*ww(1,i)*ww(1,j);  
      f = f + N(1,1:2:end)'*det(J)*ww(1,i)*ww(1,j);
      SN = SN + N*det(J)*ww(1,i)*ww(1,j);
     end
  end    
end
